import Global from 'nexie';

const theme = '@wayang/theme';

abstract class Style {
	constructor() {
		const id = this.getId();
		const key = `@wayang/style/${id}`;
		const exists = Global.has(key);
		if (exists) {
			throw new Error(`A style with an id of '${id}' already exists.`);
		}

		Global.set(key, this);
	}

	public abstract readonly id: string;
	public abstract readonly prefetch: boolean;
	public abstract async content(): Promise<string>;

	public static get theme() {
		return Global.get(theme);
	}
	public static set theme(name: string) {
		Global.upsert(theme, name);
	}

	private getId(): string {
		return this.id;
	}
}

export default Style;

// example
// class ButtonTheme extends Style {
//     public readonly id = 'button-theme';
//     public readonly prefetch = true;
//     public async content(): Promise<string> {
//         const request = await fetch(`./button/${ Style.theme }`);
//         return request.text();
//     }
// }
